@@ -9,9 +9,9 @@
 # Created: 2000.Jan.19 by Jeremy Wadsack for Wadsack-Allen Digital Group
 # 	Copyright (C) 2000 Wadsack-Allen. All rights reserved.
 ############################################################
-# Date		Modification				Author
+# Date      Modification                               Author
 # ----------------------------------------------------------
-#
+# 2000APR18 Modified to be compatible w/ GD::Graph 1.30  JAW
 ############################################################
 package GD::Graph::pie3d;
 
@@ -22,7 +22,7 @@ use GD::Graph::pie;
 use Carp;
 
 @GD::Graph::pie3d::ISA = qw( GD::Graph::pie );
-$GD::Graph::pie3d::VERSION = '0.34';
+$GD::Graph::pie3d::VERSION = '0.40';
 
 my %Defaults = (
 	'3d'        => 1,
@@ -30,10 +30,11 @@ my %Defaults = (
 
 sub initialise {
 	my $self = shift;
-	$self->SUPER::initialise();
+	my $rc = $self->SUPER::initialise();
 	while( my($key, $val) = each %Defaults ) { 
 		$self->{$key} = $val;
 	} # end while
+	return $rc;
 } # end initialise
 
 # Inherit everything else from GD::Graph::pie
